import re
import sys
from functools import update_wrapper
from django.http import HttpResponse
from django.utils import simplejson


uncamel_patterns = (
    re.compile('(.)([A-Z][a-z]+)'),
    re.compile('([a-z0-9])([A-Z])'),
    )


class HttpJSONResponse(HttpResponse):
    """
    A convenient response class for json serializable data.
    """
    def __init__(self, content='', mimetype=None, **kwargs):
        content = simplejson.dumps(content)
        mimetype = mimetype or 'application/json'
        super(HttpJSONResponse, self).__init__(
            content=content, mimetype=mimetype, **kwargs
            )


def uncamel(s):
    """
    Make camelcase lowercase and use underscores.

        >>> uncamel('CamelCase')
        'camel_case'
        >>> uncamel('CamelCamelCase')
        'camel_camel_case'
        >>> uncamel('Camel2Camel2Case')
        'camel2_camel2_case'
        >>> uncamel('getHTTPResponseCode')
        'get_http_response_code'
        >>> uncamel('get2HTTPResponseCode')
        'get2_http_response_code'
        >>> uncamel('HTTPResponseCode')
        'http_response_code'
        >>> uncamel('HTTPResponseCodeXYZ')
        'http_response_code_xyz'
    """
    for pat in uncamel_patterns:
        s = pat.sub(r'\1_\2', s)
    return s.lower()


class _Missing(object):
    def __repr__(self):
        return 'no value'

    def __reduce__(self):
        return '_missing'

_missing = _Missing()


def import_string(import_name, silent=False):
    """Imports an object based on a string.  This is useful if you want to
    use import paths as endpoints or something similar.  An import path can
    be specified either in dotted notation (``xml.sax.saxutils.escape``)
    or with a colon as object delimiter (``xml.sax.saxutils:escape``).

    If `silent` is True the return value will be `None` if the import fails.

    :param import_name: the dotted name for the object to import.
    :param silent: if set to `True` import errors are ignored and
                   `None` is returned instead.
    :return: imported object
    """
    # force the import name to automatically convert to strings
    if isinstance(import_name, unicode):
        import_name = str(import_name)
    try:
        if ':' in import_name:
            module, obj = import_name.split(':', 1)
        elif '.' in import_name:
            module, obj = import_name.rsplit('.', 1)
        else:
            return __import__(import_name)
        # __import__ is not able to handle unicode strings in the fromlist
        # if the module is a package
        if isinstance(obj, unicode):
            obj = obj.encode('utf-8')
        try:
            return getattr(__import__(module, None, None, [obj]), obj)
        except (ImportError, AttributeError):
            # support importing modules not yet set up by the parent module
            # (or package for that matter)
            modname = module + '.' + obj
            try:
                __import__(modname)
            except ImportError, e:
                raise ImportError('Failed to import %s: %s' % (modname, e))
            return sys.modules[modname]
    except ImportError:
        if not silent:
            raise


class cached_property(object):
    """A decorator that converts a function into a lazy property. The
    function wrapped is called the first time to retrieve the result
    and then that calculated result is used the next time you access
    the value::

        class Foo(object):

            @cached_property
            def foo(self):
                # calculate something important here
                return 42

    The class has to have a `__dict__` in order for this property to
    work.
    """

    # implementation detail: this property is implemented as non-data
    # descriptor. non-data descriptors are only invoked if there is
    # no entry with the same name in the instance's __dict__.
    # this allows us to completely get rid of the access function call
    # overhead. If one choses to invoke __get__ by hand the property
    # will still work as expected because the lookup logic is replicated
    # in __get__ for manual invocation.

    def __init__(self, func):
        self.func = func
        update_wrapper(self, func)

    def __get__(self, obj, owner):
        if obj is None:
            return self
        value = obj.__dict__.get(self.__name__, _missing)
        if value is _missing:
            value = self.func(obj)
            obj.__dict__[self.__name__] = value
        return value





from django.shortcuts import render_to_response
from django.template import RequestContext
from utkik.decorators import http_methods
from utkik.utils import uncamel


class ContextData(object):
    """
    A container for attributes to store on the template context.

    All the attributes are later collected as a dictionary via ``__dict__``.
    """


class View(object):
    """
    aino-utkik Goals
    ----------------
    - Building context for rendering should be simple.

    - Source should be easy to follow and encourage this for implementing
      subclasses if possible.

    - Keep methods short and provide useful hooks for sub classing.

    - Embrace the instance and don't pass request nor context around.

    - Narrow the scope to most common use but without limiting less usual
      use-cases.

    - We don't like super. We want common View subclasses without the need for
      super.

    - Strong convention over configuration, close to magic.
    """
    methods = ['GET', 'POST', 'PUT', 'DELETE'] # allowed HTTP methods
    decorators = [] # a list of decorators
    template_name = None # template name to render to
    ajax_template_name = None # template name to render to for ajax calls

    def __init__(self):
        """
        Create a :class:`ContextData` instance for the view.
        """
        self.c = ContextData() # c is for context
        self.request = None

    def dispatch(self, request, *args, **kwargs):
        """
        View entry point.

        The utkik dispatcher will create a new instance of the current class
        and call this method when the Django handler makes a call to the view.
        """
        self.request = request
        return self._decorate(self.get_response)(request, *args, **kwargs)

    def _decorate(self, f):
        """
        Decorate a function with decorators from :attr:`decorators` and
        decorators based on :attr:`methods`.
        """
        for d in reversed(self.decorators):
            f = d(f)
        methods = [m for m in self.methods if hasattr(self, m.lower())]
        return http_methods(*methods)(f)

    def get_response(self, request, *args, **kwargs):
        """
        Return the response from a successful request to the view.
        Request is just passed in here for decorator compatibility.

        First :meth:`setup` is called, mostly used for context to be accessed
        across different methods. Then we get the response from suitable
        handler method based on the HTTP method call. By default the handler is
        already checked for existense in :meth:`_decorate`. If the handler does
        not return a response, :meth:`render` is called and returned.
        """
        self.setup(*args, **kwargs)
        handler = getattr(self, self.request.method.lower())
        return handler(*args, **kwargs) or self.render()

    def setup(self, *args, **kwargs):
        """
        This is where you would put code that is the same for different
        handlers. For example if you wanted to update template context data for
        both POST and GET methods in the same way::

            self.c.form = Form(data=self.request.POST or None)

        """

    def get_context_data(self):
        """
        Return a dictionary containing the context data.

        Override this method to add to or modify the context data before it is
        used to render a template.

        This is called from :meth:`render`.
        """
        return self.c.__dict__

    def get_template_names(self):
        """
        Returns list of template names to be used for the request. Used by
        :meth:`render`.
        """
        for dirname in reversed(self.__module__.split('.')):
            if dirname != 'views':
                break
        fmt = dirname, uncamel(self.__class__.__name__)
        template_names = [ self.template_name, u'%s/%s.html' % fmt ]
        if self.request.is_ajax():
            template_names = [ self.ajax_template_name,
                u'%s/%s.ajax.html' % fmt ] + template_names
        return [ t for t in template_names if t ]

    def render(self, template_name=None):
        """
        Renders :meth:`self.get_template_names` using :meth:`get_context_data`,
        alternatively renders a provided `template_name` template.

        By default, this is called from :meth:`get_response` if the handler does
        not return a response.
        """
        return render_to_response(template_name or self.get_template_names(),
            self.get_context_data(), RequestContext(self.request))




import re
import sys
from django.conf import settings
from django.core import urlresolvers
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.http import Http404
from functools import update_wrapper
from inspect import isclass
from utkik.utils import import_string, cached_property, uncamel


__all__ = ['handler404', 'handler500', 'include', 'patterns', 'url']


handler404 = 'django.views.defaults.page_not_found'
handler500 = 'django.views.defaults.server_error'


class UtkikException(Exception):
    pass


class ViewWrapper(object):
    """
    A view wrapper that makes function and class based views callable
    """

    def __init__(self, view):
        self.view = view
        if not hasattr(view, '__name__') and isinstance(view, object):
            # for class instances that use call
            view.__name__ = view.__class__.__name__
        update_wrapper(self, view)

    @property
    def func_name(self):
        """
        This is just a lame hack to get decent debug info from the lame default
        handler behaviour.
        """
        return self.__name__

    def __call__(self, request, *args, **kwargs):
        """
        In case of the wrapped view being determined as a class (that it has
        a dispatch attribute) we return a new instance of the class with all
        view arguments passed to the dispatch method. The case of a view
        function things are much simpler, we just call the view function with
        the view arguments.

        For debugging purposes we insert some additional information that is
        useful for view classes in the raised exception.
        """
        try:
            if isclass(self.view):
                view = self.view()
                if hasattr(view, 'dispatch'):
                    return view.dispatch(request, *args, **kwargs)
            else:
                view = self.view
            if callable(view):
                return view(request, *args, **kwargs)
        except (Http404, PermissionDenied, SystemExit):
            raise
        except Exception, ex:
            try:
                cls, e, trace = sys.exc_info()
                msg = '%s in %s.%s: %s' % (
                    cls.__name__, self.view.__module__, self.view.__name__, e
                    )
            except Exception:
                raise ex
            else:
                raise UtkikException(msg), None, trace
        raise ImproperlyConfigured('%s.%s does not define a view function or '
            'class view.' % (self.view.__module__, self.view.__name__))

    def __getattr__(self, name):
        return getattr(self.view, name)


class LazyViewWrapper(ViewWrapper):
    """
    Lazy import wrapper for a view function or class.
    """

    def __init__(self, dot_name):
        module, name = dot_name.rsplit('.', 1)
        if module in settings.INSTALLED_APPS:
            module += '.views'
        self.__module__ = module
        self.__name__ = name
        self.dot_name = '%s.%s' % (module, name)

    @cached_property
    def view(self):
        """
        Return and cache the view from string. Note that in the case of a
        view class that this is not an instance but the class, instantiation
        will be done in the ``__call__`` method.
        """
        view = import_string(self.dot_name)
        self.__doc__ = view.__doc__
        return view


class RegexURLPattern(urlresolvers.RegexURLPattern):
    def __init__(self, regex, callback, default_args=None, name=None):
        """
        We just changed the way the callback references are set compared to
        ``django.core.urlresolvers.RegexURLPattern.__init__``.
        """
        urlresolvers.LocaleRegexProvider.__init__(self, regex)
        self._callback = callback
        self.default_args = default_args or {}
        self.name = name

    def add_prefix(self, prefix):
        """
        Adds the prefix string to a string-based callback.
        """
        if prefix and isinstance(self._callback, basestring):
            self._callback = '%s.%s' % (prefix, self._callback)

    @cached_property
    def callback(self):
        """
        This method is a little different from the default
        ``django.core.urlresolvers.RegexURLPattern.callback`` in that we return
        the callback wrapped in a ``ViewWrapper`` or ``LazyViewWrapper``.
        """
        try:
            if isinstance(self._callback, basestring):
                return LazyViewWrapper(self._callback)
            return ViewWrapper(self._callback)
        except Exception, e:
            raise urlresolvers.ViewDoesNotExist(e)


def include(arg, namespace=None, app_name=None):
    """
    Used to include another urls pattern file
    """
    if isinstance(arg, tuple):
        # callable returning a namespace hint
        if namespace:
            raise ImproperlyConfigured(
                'Cannot override the namespace for a dynamic module that '
                'provides a namespace'
                )
        urlconf_module, app_name, namespace = arg
    else:
        # No namespace hint - use manually provided namespace
        urlconf_module = arg
    return (urlconf_module, app_name, namespace)


def patterns(prefix, *args):
    """
    Container for url regexp patterns
    """
    pattern_list = []
    for t in args:
        if isinstance(t, (list, tuple)):
            t = url(prefix=prefix, *t)
        elif isinstance(t, (RegexURLPattern, urlresolvers.RegexURLPattern)):
            t.add_prefix(prefix)
        pattern_list.append(t)
    return pattern_list


def url(regex, view, kwargs=None, name=None, prefix=''):
    """
    sets up a regexp pattern for url processing
    """
    if isinstance(view, (list, tuple)):
        # For include(...) processing.
        urlconf_module, app_name, namespace = view
        return urlresolvers.RegexURLResolver(regex, urlconf_module, kwargs,
            app_name=app_name, namespace=namespace
            )
    else:
        if isinstance(view, basestring):
            if not view:
                raise ImproperlyConfigured('Empty URL pattern view name not '
                    'permitted (for pattern %r)' % regex
                    )
            if prefix:
                view = '%s.%s' % (prefix, view)
            if name is None:
                name = uncamel(view.split('.')[-1])
        return RegexURLPattern(regex, view, kwargs, name)




from django.http import HttpResponse
from functools import wraps


def requires_ajax(f):
    """Enforces that the request is an ajax call"""
    @wraps(f)
    def wrapper(request, *args, **kwargs):
        if not request.is_ajax():
            return HttpResponse(status=403)
        return f(request, *args, **kwargs)
    return wrapper


def http_methods(*methods):
    """Enforces one of the supplied HTTP methods"""
    def decorator(f):
        @wraps(f)
        def wrapper(request, *args, **kwargs):
            if not request.method in methods:
                return HttpResponse(status=405)
            return f(request, *args, **kwargs)
        return wrapper
    decorator.__name__ = 'http_methods'
    return decorator


def handler_decorator(*decorators):
    """Converts function decorators into a decorator for ``utkik.View``
    handlers.
    """
    def decorator(f):
        @wraps(f)
        def wrapper(self, *args, **kwargs):
            def g(request, *args, **kwargs):
                return f(self, *args, **kwargs)
            for d in reversed(decorators):
                g = d(g)
            return g(self.request, *args, **kwargs)
        return wrapper
    decorator.__name__ = 'handler_decorator'
    return decorator

